
/********************************* 
GAUSS code for reproducing 

VECM(2) Model for U.S. Output-Consumption Growth in Section 4 of  

Yunjong Eo and James Morley (2014)
"Likelihood-Ratio-Based Confidence Sets for the Timing of Structural Breaks"

lastly revised on Feb 24 2014
*********************************/

new;
library pgraph;
pqgwin many;

printout = 1;

format /ld 6,3;
cls;

@select your output file@

output file = vecm_cs.out reset; 

@Load the data@
         
load data[261,2]=ycdata.txt;                  

data = 400*ln(data);

@Specify the number of observations@

T=258;  

@Set your T by n matrix of dependent variables@                                  

y=data[4:T+3,1:2]-data[3:T+2,1:2];  n = cols(y);

@Set your T by q matrix of regressors@  
                        
z=ones(T,1) ~ (data[3:T+2,2]-data[3:T+2,1]) ~ (data[3:T+2,1:2]-data[2:T+1,1:2]) ~ (data[2:T+1,1:2]-data[1:T,1:2]) ;     

@Specify the matrix _S if necessary. 
If all the regressors are used, _S = I(p) where p = n*q, q =cols(z) and n = # of equations. See Eo and Morley (2014) for more details.@                   

_S = eye(n*cols(z));
                                                                      
@Set the estimated number of breaks m. Note that Qu and Perron's (2007) code reports confidence intervals for the maximum number of breaks.
To compare to their confidence sets, m needs to be set to the estimated number of breaks in their code too.@

m=4;

@If you have restrictions, specify the matrix R. 
if you do not have restrictions, set R=eye(cols(_S)*(m+1))@

R=eye(cols(_S)*(m+1));
                                                                           
        
@Set the trimming parameter that specifies the minimal length of a segment as a proportion of the sample size.
If the estimated break dates do not satisfy the minimal length constraint, confidence sets are not produced.@
              
trm=0.15;                  

@set brv=1 when allowing breaks in the covariance matrix of the errors; otherwise, set brv=0@
                       

brv=1;   

@set brbeta=1 when allowing breaks in regression coefficients; otherwise, set brbeta=0.@                  
                            
brbeta=1;    

@set vauto=1 when applying a correction for heteroskedasticity and/or serial correlation in the errors
(in which case Andrews' (1991) method is used to construct the robust 
covariance matrix. @
         
         
vauto=0;   

@set prewhit=1 (with vauto=1) if you want to apply an AR(1) pre-whitening when estimating the 
robust covariance matrix (see Andrews and Monahan (1992). @
                
prewhit=0;                 

@Set hetq=1 if you want to allow the distribution of the regressors to change across
regimes (this is used only when constructing confidence intervals for the estimates 
of the break dates. For the construction of the test hetq=1 always.@

hetq=1;                    


/* Constructing Inverted LR confidence sets for break dates */


@Enter in estimates of break dates in vector br based on output from Qu and Perron code or elsewhere@

br = 56.000|101.000|141.000|194.000;

@Calculate ILR confidence sets for break dates @

{csm99, csm95, csm90, kappam, rbetast, rvvst}=invLR_cs(y,z,m,T,br,hetq,vauto,brv,brbeta,prewhit);

@Output for the main procedure 

csm: confidence sets matrix (T x m), 
     if the (t,j)-th element of the matrix is equal to 1, 
     the date t is included in the confidence set for the jth break date

kappam: critival value matrix (m x 1) 

rbetast: estimates of regression parameters

rvvst: estimates of variances
@

#include invLR.src;
#include mbreak.src;

end;

